# encoding: utf-8
#
# Copyright (C) 2021 willipink.eu
# Author Moritz Münch moritzmuench@mailbox.org
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.


from copy import deepcopy

from django.http import HttpResponse
from django.test import TestCase, Client

from coinc.models import Coin, Country, Stamp
from coinc.views import add_coin
from coinc.helper import get_test_response_status


class TestCoin(TestCase):
    ''' all tests for the coin model '''

    def setUp(self):
        self.test_post_data = {
            'value': 1,
            'year': 2050,
            'country': 'fo',
            'stamp': '',
            'exists': 'false',
            'name': '',
            'found_by': '',
            'found_on': '03.05.2040',
            'circulation': 0,
            'buy_only': 'false',
            'checked': 'false',
            'ec': 'false',
#           'date-added' '':,
#           'date-modified':
        }
        Country.objects.create(
            name_iso = 'fo',
            name = 'bar'
        )
        Stamp.objects.create(
            name_short = '10 letters',
            name = 'test name'
        )


    def _return_status_wrong_field(self, field, data=None):
        ''' shortcut: return response status of post request to /coinc/add/coin '''

        test_data = deepcopy(self.test_post_data)
        if data:
            test_data[field] = data
        else:
            try:
                test_data.pop(field)
            except KeyError:
                pass
        test_response = self.client.post(
                f"/coinc/add/coin/{test_data['country']}/{test_data['year']}/{test_data['value']}",
                test_data)
        return get_test_response_status(test_response)


    def test_view_add_coin(self):

        ## value
        # value missing
#         self.assertEqual(self._return_status_wrong_field('value', None), 1)
        # value not a number
#         self.assertEqual(self._return_status_wrong_field('value', ''), 1)
#         self.assertEqual(self._return_status_wrong_field('value', 'some string'), 1)
        # value wrong number
#         self.assertEqual(self._return_status_wrong_field('value', 3), 1)
#         self.assertEqual(self._return_status_wrong_field('value', 0.02), 1)
#         self.assertEqual(self._return_status_wrong_field('value', -3), 1)
        # value ok
        for value in [1, 2, 5, 10, 20, 50, 100, 200, 201, 202, 203]:
            self.assertEqual(self._return_status_wrong_field('value', value), 0)

        ## year
        # year missing
#         self.assertEqual(self._return_status_wrong_field('year', None), 1)
        # year not a number self.assertEqual(self._return_status_wrong_field('year', ''), 1)
        # year lower limit
#         self.assertEqual(self._return_status_wrong_field('year', 1998), 1)
        # year upper limit
#         self.assertEqual(self._return_status_wrong_field('year', 2099), 1)
        # year ok
        for year in range(1999, 2098):
            self.assertEqual(self._return_status_wrong_field('year', year), 0)

        ## country
        # country missing
#         self.assertEqual(self._return_status_wrong_field('country', None), 1)
        # country not valid iso format
#         self.assertEqual(self._return_status_wrong_field('country', 12235235), 1)
#         self.assertEqual(self._return_status_wrong_field('country', 'toolong'), 1)
        # country does not exist
        self.assertEqual(self._return_status_wrong_field('country', 'xx'), 1)
        # country ok
        self.assertEqual(self._return_status_wrong_field('country', 'fo'), 0)


        ## stamp
        # stamp missing
        self.assertEqual(self._return_status_wrong_field('stamp', None), 0)
        # stamp title length
        self.assertEqual(self._return_status_wrong_field('stamp', '10 letters'), 0)
        self.assertEqual(self._return_status_wrong_field('stamp', '11 letters '), 1)
        # stamp does not exist
        self.assertEqual(self._return_status_wrong_field('stamp', 'xx'), 1)

        ## exists
        # missing exists
        self.assertEqual(self._return_status_wrong_field('exists', None), 0)
